/*
 *  Copyright (C) 2015  Thadeu Lima de Souza Cascardo <cascardo@minaslivre.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef _UTIL_H
#define _UTIL_H

int set_llong(char *str, long long *val);
int set_int(char **args, int argc, int *val);
int set_string(char **args, int argc, char **str);

#define SET_INT_(suffix, command, attr) \
static int run_##suffix(struct declaracao *dec, char **args, int argc) \
{ \
	int val; \
	int r = set_int(args, argc, &val); \
	if (r) \
		return r; \
	dec->attr = val; \
	return 0; \
} \
static struct cmd cmd_##suffix = { \
	.name = #command, \
	.run = run_##suffix, \
};

#define SET_INT(attr) SET_INT_(attr, attr, attr)

#define SET_STRING_(suffix, command, attr) \
static int run_##suffix(struct declaracao *dec, char **args, int argc) \
{ \
	char *val; \
	int r = set_string(args, argc, &val); \
	if (r) \
		return r; \
	dec->attr = val; \
	return 0; \
} \
static struct cmd cmd_##suffix = { \
	.name = #command, \
	.run = run_##suffix, \
}

#define SET_STRING(attr) SET_STRING_(attr, attr, attr)

static inline long long reais(long long val)
{
	return val / 100LL;
}

/* Sempre retorna um valor positivo. */
static inline int centavos(long long val)
{
	if (val > 0LL)
		return (int) (val % 100LL);
	else
		return (int) (-val % 100LL);
}

#define FMT_R "R$ %lld,%02d"
#define R(l) reais(l), centavos(l)

#endif
